/* $Id: iop-usb-int.h,v 1.23 2005/04/26 11:23:02 larsv Exp $
 *
 * ETRAX FS IOP USB 1.1 Host Controller
 *
 * Internal interfaces and constants
 *
 * Copyright (C) 2004, 2005  Axis Communications AB
 *
 * Author: Lars Viklund <larsv@axis.com>
 */

#ifndef __IOP_USB_INT_H
#define __IOP_USB_INT_H

/* time constants from the USB standard */
#define FRAME_TIME		12000	  /* in full speed bit times */
#define EOF1_EARLY		41	  /* from end of frame */
#define EOF2_NOM		10	  /* from end of frame */
#define RESET_TIME		50	  /* in frame times */

/* timer for SOF and EOF2 */
#define SOF_TMR			0
#define EOF2_TMR		1

/* timer used to count received bytes during IN transcation */
#define RX_CNT_TMR		0

/* number of errors that will cause endpoint to be disabled */
#define MAX_ERR_CNT		3

/* length of data CRC (in bytes) */
#define DATA_CRC_LEN		2

#ifndef __IOPASM__

#include <asm/arch/hwregs/dma.h>
#include <asm/arch/hwregs/intr_vect.h>

/* current configuration */
struct iop_usb_cfg {
	unsigned int		dma_map;
	unsigned int		ports;
	int			int_phy;
};
extern struct iop_usb_cfg iop_usb_cfg;

/* check if specified port is in use */
extern __inline__ int port_in_use(int port)
{
	return iop_usb_cfg.ports & (1 << port);
}

/* hardware instances used */
struct iop_usb_inst {
	unsigned int		spu;
	unsigned int		dma_in;
	unsigned int		dma_out;
	unsigned int		dmc_in;
	unsigned int		dmc_out;
	unsigned int		scrc_out;
	unsigned int		crc_par;
	unsigned int		fifo_in;
	unsigned int		fifo_out;
	unsigned int		fifo_out_extra;
	unsigned int		sw_spu;
	unsigned int		frame_timer_grp;
	unsigned int		rx_cnt_timer_grp;
};
extern const struct iop_usb_inst iop_usb_inst[];
#define INST(i)			(iop_usb_inst[iop_usb_cfg.dma_map].i)

/* transfer type descriptor (DMA group descriptor meta data) */
struct iop_usb_ttd {
	/* flags indicating traffic type */
	unsigned is_periodic :	1;
	unsigned is_isoc :	1;
	unsigned is_ctrl :	1;
	/* used when updating the periodic scedule list, CPU only */
	unsigned mark :		1;
	/* unused */
	unsigned :		12;
} __attribute__((packed));

extern __inline__ 
struct iop_usb_ttd *iop_usb_ttd(dma_descr_group *group)
{
	/* offset of md field in group descriptor */
	return (struct iop_usb_ttd *) ((void *) group + 6);
}

/* DMA context descriptor meta data (endpoint information) */
struct iop_usb_ctxt_md {
	/* md0 */
	/* unused */
	unsigned :		2;
	/* maximum time for a transaction on this ep, unit is bit times */
	unsigned maxtime :	14;
	/* md1 */
	/* device address */
	unsigned daddr :	7;
	/* endpoint number */
	unsigned epnum :	4;
	/* toggle bit */
	unsigned toggle :	1;
	/* flag indicating low speed endpoint */
	unsigned low_speed :	1;
	/* flag indicating high speed endpoint (currently unused) */
	unsigned high_speed :	1;
	/* maximum packet size for endpoint */
	unsigned maxsize :	10;
	/* status of endpoint */
	unsigned stat :		4;
	/* unused */
	unsigned :		4;
	/* md2 */
	/* pointer to input group descriptor */
	unsigned in_group :	32;
	/* md3 (reserved) */
	unsigned :		32;
	/* md4 (reserved) */
	unsigned :		32;
	/* saved_data */
	unsigned :		32;
	/* saved_data_buf */
	unsigned :		32;
} __attribute__((packed));

extern __inline__ 
struct iop_usb_ctxt_md *iop_usb_ctxt_md(dma_descr_context *context)
{
	/* offset of md0 field in context descriptor */
	return (struct iop_usb_ctxt_md *) ((void *) context + 6);
}

/* DMA data descriptor meta data (transaction information) */
struct iop_usb_data_md {
	/* type of transaction */
	unsigned type :		2;
	/* flag indicating that transaction as completed, initialized to zero */ 
	unsigned done :		1;
	/* for internal MPU code use, initialized to zero */
	unsigned zend :		1;
	/* add zero packet at end of (output) transaction */
	unsigned zpkt :		1;
	/* status of transaction, set by MPU when completed */	
	unsigned stat :		4;
	/* error count, initialized to zero */
	unsigned err_cnt :	2;
	/* unused */
	unsigned :		5;
} __attribute__((packed));

extern __inline__ 
struct iop_usb_data_md *iop_usb_data_md(dma_descr_data *data)
{
	/* offset of md field in data descriptor */
	return (struct iop_usb_data_md *) ((void *) data + 10);
}

#else /* __IOPASM__ */

/* 
 * macros for accessing fields in structs above from IOP assembly
 * offsets refer to the corresponding DMC registers
 */

#define TTD_IS_PERIODIC_BIT	(8+8+0)
#define TTD_IS_ISOC_BIT		(8+8+1)
#define TTD_IS_CTRL_BIT		(8+8+2)

#define ED_MAXTIME_LSB		(8+8+2)
#define ED_TOGGLE_BIT		11
#define ED_LOW_SPEED_BIT	12
#define ED_MAXSIZE_LSB		14
#define ED_STAT_LSB		24
#define ED_TRANS_INFO_MASK	0x1fff

#define TD_TYPE_LSB		(8+8+0)
#define TD_DONE_BIT		(8+8+2)
#define TD_ZEND_BIT		(8+8+3)
#define TD_ZPKT_BIT		(8+8+4)
#define TD_STAT_LSB		(8+8+5)
#define TD_ERR_CNT_LSB		(8+8+9)
#define TD_OUT			(TD_TYPE_LSB + 0)
#define TD_SETUP		(TD_TYPE_LSB + 1)
#define TD_TYPE_MASK		0x03
#define TD_ERR_CNT_MASK		0x03

#endif /* __IOPASM__ */

/* CPU interrupts from MPU */
#define CPU_IRQ_LAST		IOP_USB_IRQ_INTERNAL
#define CPU_IRQ_MASK		(~(~0 << (CPU_IRQ_LAST + 1)))

/* GIO */
#define GIO_PORT_OFFSET		4	/* offset between ports */
#define GIO_V_MO		0
#define GIO_V_PO		1
#define GIO_SE0_N		2	/* V_M or:ed with V_P in SAP_IN */
#define GIO_V_P			3
#define GIO_OE			16
#define GIO_SPEED		17
#define GIO_RCV			18

/* order of port pins */
#define PORT0_IDX		2
#define PORT1_IDX		3
#define PORT2_IDX		1
#define PORT3_IDX		0
#define GIO_PORT_BASE(port)	(PORT##port##_IDX * GIO_PORT_OFFSET)

/* GIO-bits for port */
#define PORT_GIO_OE(n)		((1<<GIO_OE)<<GIO_PORT_BASE(n))
#define PORT_GIO_SPEED(n)	((1<<GIO_SPEED)<<GIO_PORT_BASE(n))

/* all output GIOs for port */
#define GIO_OUT_ALL		(1<<GIO_V_MO|1<<GIO_V_PO|1<<GIO_OE|1<<GIO_SPEED)
#define PORT_GIO_OUT_ALL(n)	(GIO_OUT_ALL<<GIO_PORT_BASE(n))

/* trigger groups for ports */	
#define PORT0_TRIGGER_GRP	iop_trigger_grp2
#define PORT1_TRIGGER_GRP	iop_trigger_grp3
#define PORT2_TRIGGER_GRP	iop_trigger_grp1
#define PORT3_TRIGGER_GRP	iop_trigger_grp0

/* timer groups for ports */	
#define PORT0_TIMER_GRP		iop_timer_grp2
#define PORT1_TIMER_GRP		iop_timer_grp3
#define PORT2_TIMER_GRP		iop_timer_grp1
#define PORT3_TIMER_GRP		iop_timer_grp0

/* MPU addresses */
#define MPU_ADDR_START		0x000
#define MPU_ADDR_READ_MEM	0x008
#define MPU_ADDR_DATA_BASE	0x020
#define MPU_ADDR_FW_ID		(MPU_ADDR_DATA_BASE + 0)
#define MPU_ADDR_FRAME_NUMBER	(MPU_ADDR_DATA_BASE + 4)
#define MPU_ADDR_PORTS		(MPU_ADDR_DATA_BASE + 8)
#define MPU_ADDR_RH_PORT_REQ	(MPU_ADDR_DATA_BASE + 12)
#define MPU_ADDR_FE_IRQ_REQ	(MPU_ADDR_DATA_BASE + 16)
#define MPU_ADDR_PORT_STAT_0	(MPU_ADDR_DATA_BASE + 20)

/* layout of per port data in MPU memory */
#define MPU_DATA_PORT_SHIFT	(3+2)
#define MPU_DATA_PORT_SIZE	(1 << MPU_DATA_PORT_SHIFT)

#ifndef __IOPASM__

/* MPU memory address of port status register for specified port */
extern __inline__ unsigned mpu_addr_port_stat(int port)
{
	return MPU_ADDR_PORT_STAT_0 + port * MPU_DATA_PORT_SIZE;
}

#endif /* __IOPASM__ */

/* SPU addresses */
#define SPU_ADDR_START		0x000

/* MPU general registers */
#define MPU_REG_DATA		0	/* scratch register for CPU */

/* SPU general registers */
#define SPU_REG_TRANS_INFO	0	/* transaction info from MPU */
#define SPU_REG_TRANS_STATUS	1	/* transaction status to MPU */
#define SPU_REG_FRAME_NUMBER	2	/* current frame number */
#define SPU_REG_PORT_FULL_SPEED 3	/* full speed ports, updated by MPU */
#define SPU_REG_PORT_LOW_SPEED	4	/* low speed ports, updated by MPU */

/* macros for referencing registers in assembly code */
#define XCONCAT(x, n)		x##n
#define CONCAT(x, n)		XCONCAT(x, n)
#define REG(n)			XCONCAT(r, n)

/* SPU register with inputs sampled at SOF */
#ifdef __SPU__
#define GIO_IN_SOF		b0out
#endif /* __SPU__ */
#ifdef __MPU__
#define GIO_IN_SOF		REG_ADDR(iop_spu, SPU, rw_bus0_out)
#endif /* __MPU__ */

/* to speed up simulation */
#ifdef IOP_USB_QUICK_TEST
#undef FRAME_TIME
#define FRAME_TIME		2400
#undef RESET_TIME
#define RESET_TIME		5
#endif /* SIMDEF_TARGET_BANANA */

/* 
 * Format of CPU_DATA register (CPU->MPU)
 *   [ 3: 0] request
 *   [ 7: 4] port
 *   [31: 8] feature bits
 */
#define CPU_DATA_REQUEST_LSB	0
#define CPU_DATA_PORT_LSB	4
#define CPU_DATA_PORT_MASK	0xf
#define CPU_DATA_FEATURE_LSB	8

#define CPU_DATA_VAL(req, port, feat) \
	((1 << ((feat) + CPU_DATA_FEATURE_LSB)) | \
	 ((port) << CPU_DATA_PORT_LSB) | \
	 ((req) << CPU_DATA_REQUEST_LSB))

#ifdef __IOPASM__

/* MPU interrupts from SPU */
#define MPU_IRQ_SOF		2	/* start of frame sent */
#define MPU_IRQ_DONE		3	/* transaction completed */

/* 
 * Format of transaction information (MPU->SPU)
 *   [ 6: 0] USB device address 
 *   [10: 7] USB end-point address
 *   [11:11] data toggle bit
 *   [12:12] transaction is low speed
 *   [13:13] transaction is Isochronous
 *   [15:14] transfer type
 */
#define TRANS_INFO_DATA_TOGGLE	11
#define TRANS_INFO_LOW_SPEED	12
#define TRANS_INFO_IS_ISOC	13
#define TRANS_INFO_TYPE_LSB	14
#define TRANS_INFO_OUT		(TRANS_INFO_TYPE_LSB + 0)
#define TRANS_INFO_SETUP	(TRANS_INFO_TYPE_LSB + 1)

/* Transaction status (SPU->MPU) */
#define TRANS_STAT_OK		0x0000
#define TRANS_STAT_RETRY_BIT	0
#define TRANS_STAT_STALL_BIT	1
#define TRANS_STAT_BABBLE_BIT	2
#define TRANS_STAT_E_LSB	3
#define TRANS_STAT_RETRY	(1 << TRANS_STAT_RETRY_BIT)
#define TRANS_STAT_STALL	(1 << TRANS_STAT_STALL_BIT)
#define TRANS_STAT_BABBLE	(1 << TRANS_STAT_BABBLE_BIT)
#define TRANS_STAT_E_BITSTUFF	(IOP_USB_STAT_E_BITSTUFF << TRANS_STAT_E_LSB)
#define TRANS_STAT_E_PID_CHK	(IOP_USB_STAT_E_PID_CHK	 << TRANS_STAT_E_LSB)
#define TRANS_STAT_E_CRC	(IOP_USB_STAT_E_CRC	 << TRANS_STAT_E_LSB)
#define TRANS_STAT_E_BUS_TO	(IOP_USB_STAT_E_BUS_TO	 << TRANS_STAT_E_LSB)

#endif /* __IOPASM__ */

/* firmware ID */

#define FW_ID_MAJOR_VERSION	0
#define FW_ID_MINOR_VERSION	1

#define FW_ID_MAGIC_LSB		24
#define FW_ID_MAGIC_MASK	0xff000000
#define FW_ID_MAGIC		(0x42 << FW_ID_MAGIC_LSB)
#define FW_ID_PROC_BIT		23
#define FW_ID_PROC_MASK		(1 << FW_ID_PROC_BIT)
#define FW_ID_PROC_SPU		(0 << FW_ID_PROC_BIT)
#define FW_ID_PROC_MPU		(1 << FW_ID_PROC_BIT)
#define FW_ID_DMAMAP_BIT	22
#define FW_ID_DMAMAP_MASK	(1 << FW_ID_DMAMAP_BIT)
#define FW_ID_MAJOR_LSB		8
#define FW_ID_MAJOR_MASK	0x03f0
#define FW_ID_MINOR_LSB		0
#define FW_ID_MINOR_MASK	0x000f

#define FW_ID(p, d) \
	(FW_ID_MAGIC | FW_ID_PROC_##p | (d) << FW_ID_DMAMAP_BIT | \
	 FW_ID_MAJOR_VERSION << FW_ID_MAJOR_LSB | \
	 FW_ID_MINOR_VERSION << FW_ID_MINOR_LSB)

#define FW_ID_MAJOR(id)		(((id) & FW_ID_MAJOR_MASK) >> FW_ID_MAJOR_LSB)
#define FW_ID_MINOR(id)		(((id) & FW_ID_MINOR_MASK) >> FW_ID_MINOR_LSB)

#define CHECK_FW_ID_MAGIC(id)	(((id) & FW_ID_MAGIC_MASK) == FW_ID_MAGIC)
#define CHECK_FW_ID_PROC(id, p) (((id) & FW_ID_PROC_MASK) == FW_ID_PROC_##p)
#define CHECK_FW_ID_DMAMAP(id, d) \
	(((id) & FW_ID_DMAMAP_MASK) == (d) << FW_ID_DMAMAP_BIT)

#define CHECK_FW_ID(id, p, d) \
	(CHECK_FW_ID_MAGIC(id) && \
	 CHECK_FW_ID_PROC(id, p) && \
	 CHECK_FW_ID_DMAMAP(id, d) && \
	 (FW_ID_MAJOR(id) == FW_ID_MAJOR_VERSION))

#endif /* __IOP_USB_INT_H */
