/** @file
 *
 * ETRAX FS IOP USB 1.1 Host Controller
 *
 * Interface to Host Controller Driver
 *
 * Copyright (C) 2004, 2005  Axis Communications AB
 *
 * @author  Lars Viklund <larsv@axis.com>
 * @version $Id: iop-usb.h,v 1.16 2005/05/11 09:47:50 larsv Exp $
 */

#ifndef __IOP_USB_H
#define __IOP_USB_H

#ifndef __IOPASM__

#include "compat.h"
#include <asm/arch/hwregs/dma.h>

/**
 * Number of root hub ports
 */
#define IOP_USB_ROOT_HUB_PORTS	4

/**
 * Transfer types 
 */
enum iop_usb_transfer_type {
	IOP_USB_ISOC =		0, /**< Isochronous Transfer		*/
	IOP_USB_INTR =		1, /**< Interrupt Transfer		*/
	IOP_USB_CTRL =		2, /**< Control Transfer		*/
	IOP_USB_BULK =		3, /**< Bulk Transfer			*/
};
/* 
 * Both the low level CPU code and the MPU code depends on the above
 * numbering. Do not change it unless you are sure you know what you
 * are doing. 
 */

/**
 * Transaction types 
 */
enum iop_usb_transaction_type {
	IOP_USB_IN =		0, /**< IN Transaction			*/
	IOP_USB_OUT =		1, /**< OUT Transaction			*/
	IOP_USB_SETUP =		3, /**< SETUP Transaction		*/
};

/**
 * Speeds 
 */
enum iop_usb_speed {
	IOP_USB_SPEED_UNKNOWN = 0, /**< Unknown Speed			*/
	IOP_USB_SPEED_LOW =	1, /**< Low Speed			*/
	IOP_USB_SPEED_FULL =	2, /**< Full Speed			*/
	IOP_USB_SPEED_HIGH =	3, /**< High Speed			*/
};

/**
 * Return values
 */
enum iop_usb_errors {
	IOP_USB_OK	      = 0, /**< No Error			*/
	IOP_USB_ERR_INVAL     = 1, /**< Invalid input			*/
	IOP_USB_ERR_BAD_FW    = 2, /**< Bad firmware loaded		*/
	IOP_USB_ERR_BUSY      = 3, /**< Required resource busy		*/
	IOP_USB_ERR_TIMEOUT   = 4, /**< Timout when talking to IOP	*/
	IOP_USB_ERR_CTXT_DIS  = 5, /**< Context disabled		*/
	IOP_USB_ERR_ALL_DONE  = 6, /**< All descriptors already done	*/
};

/* Alignment of descriptors. */
#define ALIGN_GROUP_DESCR	16
#define ALIGN_CTXT_DESCR	32
#define ALIGN_DATA_DESCR	16
#define ALIGN_IOP_USB_ED	ALIGN_CTXT_DESCR
#define ALIGN_IOP_USB_TD	ALIGN_CTXT_DESCR
#define ALIGNED(d)		__attribute__((aligned(ALIGN_##d)))

/**
 * Endpoint descriptor
 *
 * Content is opaque. For output endpoints (not control) the last two
 * fields are not used.
 */
struct iop_usb_ed {
	dma_descr_context	ctxt			ALIGNED(CTXT_DESCR);
	u8			type;
	u8			is_in;
	u16			interval;
	struct iop_usb_td      *head_out;
	dma_descr_context      *head_in;
	dma_descr_group		in_group		ALIGNED(GROUP_DESCR);
};

/**
 * Transaction descriptor
 *
 * Content of u.priv is opaque. For OUT and SETUP transactions only
 * the first field in u.priv is used.
 */
struct iop_usb_td {
	union {
		struct iop_usb_td *next;
		struct {
			dma_descr_data		d_out	ALIGNED(DATA_DESCR);
			dma_descr_data		d_in	ALIGNED(DATA_DESCR);
			dma_descr_context	c_in	ALIGNED(CTXT_DESCR);
			dma_descr_data		dummy	ALIGNED(DATA_DESCR);
		} priv;
	} u;
};

/**
 * Initialize multiplexing of DMA channels and pins, start clock
 *
 * Should be called before other functions in this unit.
 *
 * @param dma_map	DMA mapping to use (0 or 1)
 * @param ports		Ports to use (bitmap)
 * @param int_phy	Use internal PHY for specified port (-1, 0 or 1)
 * @return		IOP_USB_OK, IOP_USB_ERR_INVAL or IOP_USB_ERR_BUSY
 */
int iop_usb_init(unsigned int dma_map, unsigned int ports, int int_phy);

/**
 * Free resources allocated by iop_usb_init()
 */
void iop_usb_exit(void);

/**
 * Configure IOP
 */
void iop_usb_config(void);

/**
 * Reset processors firmware and driver
 *
 * @return		IOP_USB_OK or IOP_USB_ERR_TIMEOUT
 */
int iop_usb_reset(void);

/**
 * Start execution
 *
 * Starts IOP and enables interrupts to CPU.
 *
 * param intr_mask	Interrupts to enable
 * @return		IOP_USB_OK, IOP_USB_ERR_TIMEOUT or IOP_USB_ERR_BAD_FW
 * @pre			HCD lock held
 */
int iop_usb_start(int intr_mask);

/**
 * Get current frame number
 *
 * @return		The current frame number, or -1 if failure
 * @pre			HCD lock held
 */
int iop_usb_get_frame(void);

/**
 * Get port status and port change fields
 *
 * @param port		Port number
 * @param wPortStatus	Port status (output)
 * @param wPortChange	Port status change (output)
 * @return		IOP_USB_OK, IOP_USB_ERR_INVAL or IOP_USB_ERR_TIMEOUT
 * @pre			HCD lock held
 */
int iop_usb_port_stat(unsigned int port, u16 *wPortStatus, u16 *wPortChange);

/**
 * Set port status feature
 *
 * @param port		Port number
 * @param feat		Port status feature to set
 * @return		IOP_USB_OK, IOP_USB_ERR_INVAL or IOP_USB_ERR_TIMEOUT
 * @pre			HCD lock held
 */
int iop_usb_set_port_feat(unsigned int port, unsigned int feat);

/**
 * Clear port status or port status change feature
 *
 * @param port		Port number
 * @param feat		Port status or port status change feature to clear
 * @return		IOP_USB_OK, IOP_USB_ERR_INVAL or IOP_USB_ERR_TIMEOUT
 * @pre			HCD lock held
 */
int iop_usb_clr_port_feat(unsigned int port, unsigned int feat);

/**
 * Initialize a descriptor for an endpoint
 *
 * @param ed		Pointer to endpoint descriptor
 * @param type		Transfer type
 * @param daddr		Destination address
 * @param epnum		Endpoint number
 * @param is_in		Is input?
 * @param speed		Speed
 * @param maxsize	Maximum packet size
 * @param interval	Interval (for ISOC and INTR)
 * @param maxtime	Maximum transfer time (in ns)
 */
void iop_usb_endpoint_init(struct iop_usb_ed *ed, 
			   enum iop_usb_transfer_type type, 
			   unsigned int daddr, 
			   unsigned int epnum, 
			   int is_in, 
			   enum iop_usb_speed speed, 
			   unsigned int maxsize, 
			   unsigned int interval, 
			   unsigned long maxtime);

/**
 * Add a descriptor for an endpoint
 *
 * @param ed		Pointer to endpoint descriptor
 * @pre			Interrupts blocked
 * @pre			HCD lock held
 */
void iop_usb_endpoint_add(struct iop_usb_ed *ed);

/**
 * Release a descriptor for an endpoint
 *
 * Note that when this function returns the descriptor may still be in
 * use by the DMA. iop_usb_endpoint_release_wait() may be called to
 * wait until it is safe to free or reused the descriptor.
 *
 * @param context	Pointer to endpoint descriptor
 * @pre			Interrupts blocked
 * @pre			HCD lock held
 */
void iop_usb_endpoint_release(struct iop_usb_ed *ed);

/**
 * Wait until a release descriptor for an endpoint 
 *
 * @param context	Pointer to endpoint descriptor
 * @note		Will busy wait
 */
void iop_usb_endpoint_release_wait(struct iop_usb_ed *ed);

/**
 * Initialize a descriptor for an USB transaction
 *
 * @param td		Pointer to transaction descriptor
 * @param type		Transaction type
 * @param maxsize	Maximum packet size
 * @param buffer	Pointer to data buffer	
 * @param buffer_length Length of data buffer
 * @param zeropacket	Finish out transaction with short packet
 */
void iop_usb_trans_init(struct iop_usb_td *td,
			enum iop_usb_transaction_type type,
			unsigned int maxsize,
			void *buffer, 
			unsigned int buffer_length,
			int zeropacket);

/**
 * Enqueue descriptors for an USB transaction
 *
 * More than one transaction descriptor may be linked together. In
 * that case the next field should point to the virtual address of the
 * next descriptor.
 *
 * @param ed		Pointer to endpoint descriptor
 * @param td		Pointer to transaction descriptor
 * @return		IOP_USB_OK or IOP_USB_ERR_CTXT_DIS
 * @pre			Interrupts blocked
 * @pre			HCD lock held
 */
int iop_usb_trans_enqueue(struct iop_usb_ed *ed,
			  struct iop_usb_td *td);

/**
 * Lock an endpoint to allow dequeuing data descriptors for an USB
 * transaction
 *
 * @param ed		Pointer to endpoint descriptor
 * @post		Endpoint is locked 
 * @note		May busy wait
 */
void iop_usb_trans_dequeue_lock(struct iop_usb_ed *ed);

/**
 * Dequeue descriptors for an USB transaction
 *
 * @param ed		Pointer to endpoint descriptor
 * @param td_first	Pointer to first transaction descriptors to remove
 * @param td_last	Pointer to last transaction descriptors to remove
 * @return		IOP_USB_OK or IOP_USB_ERR_ALL_DONE
 * @pre			Interrupts blocked
 * @pre			HCD lock held
 * @invariant		Endpoint is locked
 */
int iop_usb_trans_dequeue(struct iop_usb_ed *ed,
			  struct iop_usb_td *td_first,
			  struct iop_usb_td *td_last);

/**
 * Unlock an endpoint after dequeuing data descriptors for an USB
 * transaction
 *
 * @param ed		Pointer to endpoint descriptor
 * @pre			Endpoint is locked 
 * @post		Endpoint is not locked
 */
void iop_usb_trans_dequeue_unlock(struct iop_usb_ed *ed);

/**
 * Stop processing peridic traffic of the specified type until more
 * traffic is submitted
 *
 * May be called when there is no more traffic of the specificed type
 * pending.
 *
 * @param type		Transfer type
 */
void iop_usb_stop_periodic(enum iop_usb_transfer_type type);

/**
 * Get active interrupts
 *
 * @return		Active interrupts
 * @note		Acknowledges the interrupts as a side effect
 */
int iop_usb_active_irqs(void);

/**
 * Interrupt handler for IOP_USB_IRQ_TRANS_DONE
 *
 * Processes completed transactions and calls the supplied callback
 * for each transaction descriptor.
 *
 * @param cb		Callback
 * @pre			Interrupts blocked
 * @pre			HCD lock held
 */
void iop_usb_trans_done_irq(void (*cb)(struct iop_usb_td *, int, int));

/**
 * Get SPU number used 
 *
 * @return		The SPU number (0 or 1), or -1 if failure
 */
int iop_usb_spu_nr(void);

/**
 * Get DMA mapping used by the loaded firmware
 *
 * @return		DMA mapping (0 or 1), or -1 if failure
 */
int iop_usb_dma_map(void);

#endif /* __IOPASM__ */

/*
 * Interrupt vector used
 */
#define IOP_USB_INTR_VECT	IOP0_INTR_VECT

/**
 * Interrupts (from MPU)
 */
#define IOP_USB_IRQ_PORT_STAT_C	0 /**< Port status change		 */
#define IOP_USB_IRQ_TRANS_DONE	1 /**< One or more transaction completed */
#define IOP_USB_IRQ_BAD_SCHED	2 /**< Too much periodic traffic	 */
#define IOP_USB_IRQ_INTERNAL	3 /**< Fatal internal error		 */

/*
 * Status of descriptor
 */
#define IOP_USB_STAT_OK		0 /* No Error				 */
#define IOP_USB_STAT_STALL	1 /* Stall				 */
#define IOP_USB_STAT_BABBLE	2 /* Babble				 */
#define IOP_USB_STAT_E_BITSTUFF 3 /* Bitstuff Error			 */
#define IOP_USB_STAT_E_PID_CHK	4 /* PID Check Error			 */
#define IOP_USB_STAT_E_CRC	5 /* CRC Error				 */
#define IOP_USB_STAT_E_BUS_TO	6 /* Bus Timeout			 */
#define IOP_USB_STAT_ORUN	7 /* Overrun				 */
#define IOP_USB_STAT_URUN	8 /* Underrun				 */
#define IOP_USB_STAT_FLUSHED	9 /* Flushed because endpoint halted	 */
#define IOP_USB_STAT_MSB	3

/*
 * bRequest
 */
#define IOP_USB_CLEAR_FEATURE	1
#define IOP_USB_SET_FEATURE	3

/*
 * wPortStatus (see USB 2.0 spec. section 11.24.2.7.1)
 */
#define IOP_USB_PORT_CONNECTION	0
#define IOP_USB_PORT_ENABLE	1
#define IOP_USB_PORT_SUSPEND	2
#define IOP_USB_PORT_OVERCURRENT 3
#define IOP_USB_PORT_RESET	4
#define IOP_USB_PORT_POWER	8
#define IOP_USB_PORT_LOW_SPEED	9
#define IOP_USB_PORT_HIGH_SPEED	10
#define IOP_USB_PORT_TEST	11
#define IOP_USB_PORT_INDICATOR	12

/*
 * wPortChange (see USB 2.0 spec. section 11.24.2.7.2)
 */
#define IOP_USB_C_PORT_LSB	16
#define IOP_USB_C_PORT_CONNECTION	\
	(IOP_USB_C_PORT_LSB+IOP_USB_PORT_CONNECTION)
#define IOP_USB_C_PORT_ENABLE		\
	(IOP_USB_C_PORT_LSB+IOP_USB_PORT_ENABLE)
#define IOP_USB_C_PORT_SUSPEND		\
	(IOP_USB_C_PORT_LSB+IOP_USB_PORT_SUSPEND)
#define IOP_USB_C_PORT_OVERCURRENT	\
	(IOP_USB_C_PORT_LSB+IOP_USB_PORT_OVERCURRENT)
#define IOP_USB_C_PORT_RESET		\
	(IOP_USB_C_PORT_LSB+IOP_USB_PORT_RESET)

#endif /* __IOP_USB_H */
